Attribute VB_Name = "modAPIFilesAndPaths"
Option Explicit


' Module      : modAPIFilesAndPaths
' Description : Code for working with files and paths using the Windows API
'
Public Enum EnumFileTimeType
  ftt_Created = 0
  ftt_Accessed = 1
  ftt_Modified = 2
End Enum

Private Type FILETIME
  lngLowDateTime As Long
  lngHighDateTime As Long
End Type

Private Type SYSTEMTIME
  intYear As Integer
  intMonth As Integer
  intDayOfWeek As Integer
  intDay As Integer
  intHour As Integer
  intMinute As Integer
  intSecond As Integer
  intMilliseconds As Integer
End Type

Private Const FILE_SHARE_READ = &H1
Private Const FILE_SHARE_WRITE = &H2
Private Const FILE_ATTRIBUTE_ARCHIVE = &H20
Private Const FILE_ATTRIBUTE_COMPRESSED = &H800
Private Const FILE_ATTRIBUTE_DIRECTORY = &H10
Private Const FILE_ATTRIBUTE_HIDDEN = &H2
Private Const FILE_ATTRIBUTE_NORMAL = &H80
Private Const FILE_ATTRIBUTE_READONLY = &H1
Private Const FILE_ATTRIBUTE_SYSTEM = &H4
Private Const FILE_ATTRIBUTE_TEMPORARY = &H100

Private Const OPEN_EXISTING = 3
Private Const FILE_FLAG_RANDOM_ACCESS = &H10000000
Public Const GENERIC_WRITE = &H40000000
Public Const GENERIC_READ = &H80000000


Private Declare Function CreateFile _
  Lib "kernel32" _
  Alias "CreateFileA" _
  (ByVal lpFileName As String, _
    ByVal dwDesiredAccess As Long, _
    ByVal dwShareMode As Long, _
    lpSecurityAttributes As Any, _
    ByVal dwCreationDisposition As Long, _
    ByVal dwFlagsAndAttributes As Long, _
    ByVal hTemplateFile As Long) _
  As Long

Private Declare Function CloseHandle _
  Lib "kernel32" _
  (ByVal hObject As Long) _
  As Long

    
Private Declare Function CGetFileTime _
  Lib "kernel32" _
  Alias "GetFileTime" _
  (ByVal hFile As Long, _
    lpCreationTime As FILETIME, _
    lpLastAccessTime As FILETIME, _
    lpLastWriteTime As FILETIME) _
  As Long

Private Declare Function CSetFileTime _
  Lib "kernel32" _
  Alias "SetFileTime" _
  (ByVal hFile As Long, _
    lpCreationTime As FILETIME, _
    lpLastAccessTime As FILETIME, _
    lpLastWriteTime As FILETIME) _
  As Long

Private Declare Function CompareFileTime _
  Lib "kernel32" _
  (lpFileTime1 As FILETIME, _
    lpFileTime2 As FILETIME) _
  As Long
    
Private Declare Function SystemTimeToFileTime _
  Lib "kernel32" _
  (lpSystemTime As SYSTEMTIME, _
    lpFileTime As FILETIME) _
  As Long
  
Private Declare Function FileTimeToLocalFileTime _
  Lib "kernel32" _
  (lpFileTime As FILETIME, _
    lpLocalFileTime As FILETIME) _
  As Long
    
Private Declare Function LocalFileTimeToFileTime _
  Lib "kernel32" _
  (lpLocalFileTime As FILETIME, _
    lpFileTime As FILETIME) _
    As Long
    
Private Declare Function FileTimeToSystemTime _
  Lib "kernel32" _
  (lpFileTime As FILETIME, _
    lpSystemTime As SYSTEMTIME) _
  As Long
  


Private Const MAX_PATH As Long = 260

Private Type WIN32_FIND_DATA
  dwFileAttributes As Long
  ftCreationTime As FILETIME
  ftLastAccessTime As FILETIME
  ftLastWriteTime As FILETIME
  nFileSizeHigh As Long
  nFileSizeLow As Long
  dwReserved0 As Long
  dwReserved1 As Long
  cFileName As String * MAX_PATH
  cAlternate As String * 14
End Type

Private Declare Function FindFirstFile _
  Lib "kernel32" _
  Alias "FindFirstFileA" _
  (ByVal lpFileName As String, _
    lpFindFileData As WIN32_FIND_DATA) _
  As Long

Private Declare Function GetComputerName _
  Lib "kernel32" _
  Alias "GetComputerNameA" _
  (ByVal lpBuffer As String, _
    nSize As Long) _
  As Long

Private Const MAX_COMPUTERNAME_LENGTH = 255




Private Function GetFileHandle(strFile As String) As Long
  ' Comments  : Open a file to get a handle to it. We then use
  '             this handle for other calls.
  ' Parameters: strFile - Full path and name of the file
  ' Returns   : Long integer handle or -1 if an error occurred
  
  '
  Const clngReadMode As Long = GENERIC_READ Or GENERIC_WRITE

  On Error GoTo PROC_ERR
  
  ' Call the API to get a handle
  GetFileHandle = CreateFile(strFile, clngReadMode, ByVal 0&, ByVal 0&, OPEN_EXISTING, 0&, 0&)
Debug.Print GetFileHandle
PROC_EXIT:
  Exit Function
  
PROC_ERR:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , _
    "GetFileHandle"
  Resume PROC_EXIT
        
End Function



Public Function GetFileTime( _
  strFile As String, _
  eType As EnumFileTimeType, _
  fLocalTime As Boolean) _
  As Date
  ' Comments  : Returns the specified file time for the specified file.
  ' Parameters: strFile - Full path and name of the file
  '             eType - Time to return according to then
  '             EnumFileTimeType enumerated type
  '             fLocalTime - True to return local time as defined by the
  '             current operating system settings, False to return UTC
  '             time (also known as GMT)
  ' Returns   : Date/time value
  
  '
  Dim ftFTCreate As FILETIME
  Dim ftFTAccess As FILETIME
  Dim ftFTWrite As FILETIME
  Dim lngFileHandle As Long
  Dim fOK As Boolean
  
  On Error GoTo PROC_ERR
    
  ' Get a handle to the file
  lngFileHandle = GetFileHandle(strFile)
  
  ' Make sure we got a valid handle
  If lngFileHandle > 0 Then
  
    ' Call the API to get the data
    fOK = CBool(CGetFileTime( _
      lngFileHandle, _
      ftFTCreate, _
      ftFTAccess, _
      ftFTWrite))
        
    If fOK Then
      
      ' Convert the Windows time format to something we can use
      Select Case eType
      
        Case ftt_Accessed
          GetFileTime = WinTimeToVBATime(ftFTAccess, fLocalTime)
        
        Case ftt_Modified
          GetFileTime = WinTimeToVBATime(ftFTWrite, fLocalTime)
        
        Case ftt_Created
          GetFileTime = WinTimeToVBATime(ftFTCreate, fLocalTime)
        
      End Select
      
      ' Be sure to close the file when we're done
      CloseHandle lngFileHandle
    End If
  End If
PROC_EXIT:
  Exit Function
  
PROC_ERR:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , _
    "GetFileTime"
  Resume PROC_EXIT
  
End Function



Private Function NullTerminatedStringToArray( _
  strBuf As String, _
  astrRet() As String) _
  As Integer
  ' Comments  : Converts a null-terminated string to the passed array
  ' Parameters: strBuf - null terminated string
  '             astrRet() - Set: array of strings to fill
  ' Returns   : Number of elements
  
  '
  Dim intIndex As Integer
  Dim strTmpBuf As String
  Dim intPos As Integer
  Dim intRet As Integer
  
  On Error GoTo PROC_ERR
  
  ' Assign the input value to a temporary variable
  strTmpBuf = strBuf
    
  ' Initialize the index pointer
  intIndex = -1
  
  ' Assume failure
  intRet = 0
    
  ' Loop through the input string
  Do
    intPos = InStr(strTmpBuf, vbNullChar)

    ' If we are still in bounds
    If intPos > 1 Then
      ' Grow the array
      intIndex = intIndex + 1
      ReDim Preserve astrRet(intIndex + 1)
      
      ' Assign the values
      astrRet(intIndex) = Left(strBuf, intPos - 1)
      strTmpBuf = Mid(strBuf, intPos + 1)
    End If
  Loop Until intPos <= 1
    
  ' Get the return value
  intRet = intIndex + 1
    
PROC_EXIT:
  NullTerminatedStringToArray = intRet
  Exit Function
  
PROC_ERR:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , _
    "NullTerminatedStringToArray"
  Resume PROC_EXIT
  
End Function


Public Function SetFileTime( _
  strFile As String, _
  datIn As Date, _
  eType As EnumFileTimeType) _
  As Date
  ' Comments  : Sets the specified time on the specified file
  ' Parameters: strFile - Path and name of the file to change
  '             datIn - date/time value to change to
  '             eType - which time to change as defined by the
  '             EnumFileTimeType enumerated type
  ' Returns   : New time set (should be equal to the passed time)
  
  '
  Dim ftFileTemp As FILETIME
  Dim ftFileCreated As FILETIME
  Dim ftFileAccessed As FILETIME
  Dim ftFileModified As FILETIME
  Dim lngFileHandle As Long
  Dim fOK As Boolean
  
  On Error GoTo PROC_ERR
        
  ' Get a handle to the file
  lngFileHandle = GetFileHandle(strFile)
  
  ' Make sure we got a valid handle
  If lngFileHandle > 0 Then
    
    ' Convert callers time to the correct format
    Call VBATimeToFileTime(datIn, ftFileTemp, True)
    
    ' What are the file's current date/time values?
    If CBool(CGetFileTime( _
      lngFileHandle, _
      ftFileCreated, _
      ftFileAccessed, _
      ftFileModified)) Then

      ' Decide which time to change
      Select Case eType
      
        Case ftt_Created
          ftFileCreated = ftFileTemp
        
        Case ftt_Accessed
          ftFileAccessed = ftFileTemp
        
        Case ftt_Modified
          ftFileModified = ftFileTemp
      
      End Select
      
      ' Change the date/time
      fOK = CBool(CSetFileTime( _
        lngFileHandle, _
        ftFileCreated, _
        ftFileAccessed, _
        ftFileModified))
      
      ' Set the return value
      If fOK Then
        SetFileTime = datIn
      Else
        SetFileTime = vbNull
      End If
    End If
    
    ' Close the file
    Call CloseHandle(lngFileHandle)
    
  End If
  
PROC_EXIT:
  Exit Function
  
PROC_ERR:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , _
    "SetFileTime"
  Resume PROC_EXIT
    
End Function

Private Function SystemTimeToVBATime( _
  stSystemTime As SYSTEMTIME) _
  As Date
  ' Comments  : Converts the Windows SYSTEMTIME struct to a VBA Date type
  ' Parameters: stSystemTime - Windows SYSTEMTIME struct to convert
  ' Returns   : VBA Date type
  
  '
  On Error GoTo PROC_ERR
  
  ' Use native VBA calls to build the date type
  SystemTimeToVBATime = _
    DateSerial(stSystemTime.intYear, _
    stSystemTime.intMonth, _
    stSystemTime.intDay) + _
    TimeSerial(stSystemTime.intHour, _
    stSystemTime.intMinute, _
    stSystemTime.intSecond)

PROC_EXIT:
  Exit Function
  
PROC_ERR:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , _
    "SystemTimeToVBATime"
  Resume PROC_EXIT
  
End Function

Private Sub VBATimeToFileTime( _
  datIn As Date, _
  ftFileTime As FILETIME, _
  fLocalTime As Boolean)
  ' Comments  : Converts the passed date value to the Windows FILETIME
  '             structure.
  ' Parameters: datIn - value to change
  '             ftFileTime - Windows FILETIME struct to convert into
  '             fLocalTime - True to return local time as defined by the
  '             current operating system settings, False to return UTC
  '             time (also known as GMT)
  ' Returns   : Nothing
  
  '
  On Error GoTo PROC_ERR
    
  Dim stSystemTime As SYSTEMTIME
  Dim ftFileTmp As FILETIME
  Dim fOK As Boolean
    
  ' First convert to system time
  Call VBATimeToSystemTime(datIn, stSystemTime)
    
  ' Next convert the system time to a file time
  fOK = CBool(SystemTimeToFileTime(stSystemTime, ftFileTime))
  
  ' Did it work?
  If fOK Then
    ' Handle local time
    If fLocalTime Then
      Call LocalFileTimeToFileTime(ftFileTime, ftFileTmp)
      ftFileTime = ftFileTmp
    End If
  End If
  
PROC_EXIT:
  Exit Sub
  
PROC_ERR:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , _
    "VBATimeToFileTime"
  Resume PROC_EXIT
  
End Sub

Private Sub VBATimeToSystemTime( _
  datIn As Date, _
  stSystemTime As SYSTEMTIME)
  ' Comments  : Converts the supplied VBA date/time value to the
  '             Windows SYSTEMTIME struct
  ' Parameters: datIn - date to convert
  '             stSystemTime - Windows SYSTEMTIME struct to convert into
  ' Returns   : Nothing
  
  '
  On Error GoTo PROC_ERR
  
  ' Convert using VB intrinsic functions
  With stSystemTime
    .intMonth = Month(datIn)
    .intDay = Day(datIn)
    .intYear = Year(datIn)
    .intHour = Hour(datIn)
    .intMinute = Minute(datIn)
    .intSecond = Second(datIn)
  End With
  
PROC_EXIT:
  Exit Sub
  
PROC_ERR:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , _
    "VBATimeToSystemTime"
  Resume PROC_EXIT
  
End Sub

Private Function WinTimeToVBATime( _
  ftWinFileTime As FILETIME, _
  fLocalTime As Boolean) _
  As Date
  ' Comments  : Converts the Windows FILETIME struct to a VBA Date type
  ' Parameters: ftWinFileTime - Windows FILETIME struct to convert
  '             fLocalTime - True to return local time as defined by the
  '             current operating system settings, False to return UTC
  '             time (also known as GMT)
  ' Returns   : VBA Date type
  
  '
  Dim stSystemFileTime As SYSTEMTIME
  Dim ftLocalFileTime As FILETIME
    
  On Error GoTo PROC_ERR
  
  ' Was local time requested?
  If fLocalTime Then
    ' Need to convert the file time to a local time
    FileTimeToLocalFileTime ftWinFileTime, ftLocalFileTime
    ftWinFileTime = ftLocalFileTime
  End If
  
  ' Call the API to convert the time format
  If CBool(FileTimeToSystemTime(ftWinFileTime, stSystemFileTime)) Then
    WinTimeToVBATime = SystemTimeToVBATime(stSystemFileTime)
  End If
  
PROC_EXIT:
  Exit Function
  
PROC_ERR:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , _
    "WinTimeToVBATime"
  Resume PROC_EXIT
  
End Function
